/*****************************************************************************
 * Copyright (c) 2014, 2016 CEA LIST, Christian W. Damus, and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  CEA LIST - Initial API and implementation
 *  Christian W. Damus - bug 474481
 *****************************************************************************/
package org.eclipse.papyrusrt.umlrt.core.types.advice;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.gmf.runtime.common.core.command.CommandResult;
import org.eclipse.gmf.runtime.common.core.command.CompositeCommand;
import org.eclipse.gmf.runtime.common.core.command.ICommand;
import org.eclipse.gmf.runtime.emf.type.core.commands.ConfigureElementCommand;
import org.eclipse.gmf.runtime.emf.type.core.edithelper.AbstractEditHelperAdvice;
import org.eclipse.gmf.runtime.emf.type.core.requests.ConfigureRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.CreateRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.SetRequest;
import org.eclipse.papyrus.infra.services.edit.service.ElementEditServiceUtils;
import org.eclipse.papyrus.infra.services.edit.service.IElementEditService;
import org.eclipse.papyrus.infra.services.edit.utils.RequestParameterConstants;
import org.eclipse.papyrusrt.umlrt.core.utils.NewElementUtil;
import org.eclipse.papyrusrt.umlrt.profile.UMLRealTime.CapsulePart;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.VisibilityKind;

/**
 * Edit Helper Advice for {@link CapsulePart}
 */
public class CapsulePartEditHelperAdvice extends AbstractEditHelperAdvice {

	public CapsulePartEditHelperAdvice() {
		super();
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.eclipse.gmf.runtime.emf.type.core.edithelper.AbstractEditHelperAdvice#getBeforeCreateRelationshipCommand(org.eclipse.gmf.runtime.emf.type.core.requests.CreateRelationshipRequest)
	 */
	@Override
	protected ICommand getBeforeCreateRelationshipCommand(CreateRelationshipRequest request) {
		return super.getBeforeCreateRelationshipCommand(request);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.eclipse.gmf.runtime.emf.type.core.edithelper.AbstractEditHelperAdvice#getAfterConfigureCommand(org.eclipse.gmf.runtime.emf.type.core.requests.ConfigureRequest)
	 *
	 */
	@Override
	protected ICommand getAfterConfigureCommand(ConfigureRequest request) {
		final Property capsulePart = (Property) request.getElementToConfigure();
		request.setParameter(RequestParameterConstants.DIALOG_CANCELLABLE, true);

		return new ConfigureElementCommand(request) {

			/**
			 * @see org.eclipse.gmf.runtime.emf.commands.core.command.AbstractTransactionalCommand#doExecuteWithResult(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
			 *
			 */
			@Override
			protected CommandResult doExecuteWithResult(IProgressMonitor progressMonitor, IAdaptable info) throws ExecutionException {
				capsulePart.setAggregation(AggregationKind.COMPOSITE_LITERAL);
				capsulePart.setLower(1);
				capsulePart.setUpper(1);
				capsulePart.setName(null);// set to null to let it be named when type is set
				capsulePart.setIsOrdered(true);
				capsulePart.setVisibility(VisibilityKind.PROTECTED_LITERAL);
				return CommandResult.newOKCommandResult(capsulePart);
			}

		};
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.eclipse.gmf.runtime.emf.type.core.edithelper.AbstractEditHelperAdvice#getBeforeConfigureCommand(org.eclipse.gmf.runtime.emf.type.core.requests.ConfigureRequest)
	 */
	@Override
	protected ICommand getBeforeConfigureCommand(ConfigureRequest request) {
		final Property property = (Property) request.getElementToConfigure();

		// This capsule-part is new if it is being configured
		NewElementUtil.elementCreated(property);

		CompositeCommand compositeCommand = new CompositeCommand("Before Configure CapsulePart");

		// If the newly create Port has no multiplicity set it to the default Value
		if (null == property.getLowerValue()) {
			// Set the new value based on the kind set
			LiteralInteger newLowerValue = UMLFactory.eINSTANCE.createLiteralInteger();
			newLowerValue.setValue(1);

			SetRequest setLowerBounds = new SetRequest(property, UMLPackage.eINSTANCE.getMultiplicityElement_LowerValue(), newLowerValue);
			IElementEditService commandProvider = ElementEditServiceUtils.getCommandProvider(property);
			compositeCommand.add(commandProvider.getEditCommand(setLowerBounds));

		}

		if (null == property.getUpperValue()) {
			LiteralUnlimitedNatural newUpperValue = UMLFactory.eINSTANCE.createLiteralUnlimitedNatural();
			newUpperValue.setValue(1);

			SetRequest setUpperBounds = new SetRequest(property, UMLPackage.eINSTANCE.getMultiplicityElement_UpperValue(), newUpperValue);
			IElementEditService commandProvider = ElementEditServiceUtils.getCommandProvider(property);
			compositeCommand.add(commandProvider.getEditCommand(setUpperBounds));
		}

		return compositeCommand.isEmpty() ? super.getBeforeConfigureCommand(request) : compositeCommand;
	}




}
