/*****************************************************************************
 * Copyright (c) 2016 Christian W. Damus and others.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Christian W. Damus - Initial API and implementation
 *   
 *****************************************************************************/

package org.eclipse.papyrusrt.umlrt.core.utils;

import static java.util.Spliterators.spliteratorUnknownSize;
import static java.util.stream.StreamSupport.stream;

import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Spliterator;
import java.util.stream.Stream;

import org.eclipse.emf.ecore.EObject;

/**
 * Miscellaneous utilities for working with models and model elements.
 */
public class ModelUtils {

	/**
	 * Not instantiable by clients.
	 */
	private ModelUtils() {
		super();
	}

	/**
	 * Obtains a stream over the containers of an {@code object}, not including
	 * the {@code object}, itself.
	 * 
	 * @param object
	 *            an object
	 * @return its containers, in bottom-up order
	 */
	public static Stream<EObject> containersOf(EObject object) {
		return selfAndContainersOf(object.eContainer());
	}

	/**
	 * Obtains a stream over the containers of an {@code object}, including
	 * the {@code object}, itself.
	 * 
	 * @param object
	 *            an object
	 * @return itself and its containers, in bottom-up order
	 */
	public static Stream<EObject> selfAndContainersOf(EObject object) {
		Iterator<EObject> containers = new Iterator<EObject>() {
			EObject next = object;

			@Override
			public boolean hasNext() {
				return next != null;
			}

			@Override
			public EObject next() {
				if (!hasNext()) {
					throw new NoSuchElementException();
				}

				EObject result = next;
				next = result.eContainer();
				return result;
			}
		};

		return stream(spliteratorUnknownSize(containers, Spliterator.IMMUTABLE | Spliterator.ORDERED | Spliterator.NONNULL), false);
	}
}
